/*!
 * rmux-router.js v0.0.1-alpha.15
 * (c) 2018-2019 空鱼
 * Released under the MIT License.
 */
import { Route, Router } from 'react-router-dom';
export * from 'react-router-dom';
import queryString from 'querystring';
import React, { lazy, Suspense } from 'react';
import { createBrowserHistory } from 'history';
export * from 'history';
export * from 'connected-react-router';

const defaultLoading = "loading...";
const defaultAsyncLoader = factory => lazy(factory);

const globalRouteConfig = new Map();

const LoadableComponent = ({
  loader,
  loading,
  ...props
}) => {
  const LazyComponent = (globalRouteConfig.get("asyncLoader") || defaultAsyncLoader)(loader);
  /**
   * loading 组件的优先级为 自定义 > 全局 > 默认
   */

  const fallback = loading || globalRouteConfig.get("loading") || defaultLoading;
  return React.createElement(Suspense, {
    fallback: fallback
  }, React.createElement(LazyComponent, Object.assign({}, props)));
};
const LazyRoute = props => {
  const {
    loader,
    loading
  } = props;

  const C = componentProps => {
    const parsed = queryString.parse(window.location.search.replace(/^\?/, ""));
    return React.createElement(LoadableComponent, Object.assign({}, componentProps, {
      loading: loading,
      loader: loader,
      location: { ...componentProps.location,
        query: parsed
      },
      routerParams: componentProps.match.params
    }));
  };

  return React.createElement(Route, Object.assign({}, props, {
    component: C
  }));
};
const RouterProvider = props => {
  const {
    history = createBrowserHistory(),
    children,
    loading,
    asyncLoader
  } = props;
  globalRouteConfig.set("loading", loading);
  globalRouteConfig.set("asyncLoader", asyncLoader);
  return React.createElement(Router, {
    history: history
  }, children);
};

export { LazyRoute, LoadableComponent, RouterProvider, defaultAsyncLoader, defaultLoading, globalRouteConfig };
