/*!
 * rmux-router.js v0.0.1-alpha.15
 * (c) 2018-2019 空鱼
 * Released under the MIT License.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var reactRouterDom = require('react-router-dom');
var queryString = _interopDefault(require('querystring'));
var React = require('react');
var React__default = _interopDefault(React);
var history = require('history');
var connectedReactRouter = require('connected-react-router');

const defaultLoading = "loading...";
const defaultAsyncLoader = factory => React.lazy(factory);

const globalRouteConfig = new Map();

const LoadableComponent = ({
  loader,
  loading,
  ...props
}) => {
  const LazyComponent = (globalRouteConfig.get("asyncLoader") || defaultAsyncLoader)(loader);
  /**
   * loading 组件的优先级为 自定义 > 全局 > 默认
   */

  const fallback = loading || globalRouteConfig.get("loading") || defaultLoading;
  return React__default.createElement(React.Suspense, {
    fallback: fallback
  }, React__default.createElement(LazyComponent, Object.assign({}, props)));
};
const LazyRoute = props => {
  const {
    loader,
    loading
  } = props;

  const C = componentProps => {
    const parsed = queryString.parse(window.location.search.replace(/^\?/, ""));
    return React__default.createElement(LoadableComponent, Object.assign({}, componentProps, {
      loading: loading,
      loader: loader,
      location: { ...componentProps.location,
        query: parsed
      },
      routerParams: componentProps.match.params
    }));
  };

  return React__default.createElement(reactRouterDom.Route, Object.assign({}, props, {
    component: C
  }));
};
const RouterProvider = props => {
  const {
    history: history$1 = history.createBrowserHistory(),
    children,
    loading,
    asyncLoader
  } = props;
  globalRouteConfig.set("loading", loading);
  globalRouteConfig.set("asyncLoader", asyncLoader);
  return React__default.createElement(reactRouterDom.Router, {
    history: history$1
  }, children);
};

Object.keys(reactRouterDom).forEach(function (k) {
  if (k !== 'default') Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () {
      return reactRouterDom[k];
    }
  });
});
Object.keys(history).forEach(function (k) {
  if (k !== 'default') Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () {
      return history[k];
    }
  });
});
Object.keys(connectedReactRouter).forEach(function (k) {
  if (k !== 'default') Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () {
      return connectedReactRouter[k];
    }
  });
});
exports.LazyRoute = LazyRoute;
exports.LoadableComponent = LoadableComponent;
exports.RouterProvider = RouterProvider;
exports.defaultAsyncLoader = defaultAsyncLoader;
exports.defaultLoading = defaultLoading;
exports.globalRouteConfig = globalRouteConfig;
