import { RouteComponentProps, Route, Router } from "react-router-dom";
import queryString from "querystring";
import React, { Suspense, FC } from "react";

import { createBrowserHistory } from "history";
import { LoadableProps, LazyRouteProps, RouterProviderProps } from "./@types/";
import { defaultLoading, defaultAsyncLoader } from "./constant";
import { globalRouteConfig } from "./config";

export const LoadableComponent = ({
  loader,
  loading,
  ...props
}: LoadableProps) => {
  const LazyComponent = (
    globalRouteConfig.get("asyncLoader") || defaultAsyncLoader
  )(loader);

  /**
   * loading 组件的优先级为 自定义 > 全局 > 默认
   */
  const fallback =
    loading || globalRouteConfig.get("loading") || defaultLoading;

  return (
    <Suspense fallback={fallback}>
      <LazyComponent {...props} />
    </Suspense>
  );
};
export const LazyRoute = (props: LazyRouteProps) => {
  const { loader, loading } = props;
  const C = (componentProps: RouteComponentProps) => {
    const parsed = queryString.parse(window.location.search.replace(/^\?/, ""));
    return (
      <LoadableComponent
        {...componentProps}
        loading={loading}
        loader={loader}
        location={{
          ...componentProps.location,
          query: parsed
        }}
        routerParams={componentProps.match.params}
      />
    );
  };

  return <Route {...props} component={C} />;
};

export const RouterProvider: FC<RouterProviderProps> = props => {
  const {
    history = createBrowserHistory(),
    children,
    loading,
    asyncLoader
  } = props;
  globalRouteConfig.set("loading", loading);
  globalRouteConfig.set("asyncLoader", asyncLoader);
  return <Router history={history}>{children}</Router>;
};

export * from "react-router-dom";
export * from "history";
export * from "connected-react-router";
export * from "./config";
export * from "./constant";
export * from "./@types";
